using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Security.Cryptography;
using CSockets;
using CiaoStuff;

namespace CiaoClient
{
    public partial class MainClientForm : Form
    {
        private FormWindowState PreviousWindowState = FormWindowState.Normal;
        private ClientConfiguration ThisConfiguration;
        private Client CiaoClient;
        private String Key;
        private String PublicKey;
        private String PrivateKey;

        public MainClientForm()
        {
            System.Windows.Forms.Form.CheckForIllegalCrossThreadCalls = false;
            InitializeComponent();
            CiaoClient = new Client(IPAddress.Parse("127.0.0.1"), 3724);
            this.CiaoClient.PropertyChange += new PropertyChangedEventHandler(CiaoClient_PropertyChange);
            this.CiaoClient.PacketTransmit += new PacketEventHandler(CiaoClient_PacketTransmit);
            LoadConfiguration();
        }

        private void MainClientForm_Load(object sender, EventArgs e)
        {
            this.tabControl1.SelectedIndex = 1;
            this.tabControl1.SelectedIndex = 0;
        }

        void CiaoClient_PacketTransmit(object sender, PacketEventArgs e)
        {
            if (e.Direction == PacketDirection.Incoming)
            {
                OnPacketRceived(e.ReceivedPacket);
            }
        }

        private void LoadConfiguration()
        {
            ThisConfiguration = (ClientConfiguration)Serializer.LoadObject("conf.xml", Serializer.FormatterType.SOAP);
            if (ThisConfiguration == null)
            {
                ThisConfiguration = new ClientConfiguration();
                Serializer.SaveObject(ThisConfiguration, "conf.xml", Serializer.FormatterType.SOAP);
            }
        }

        void CiaoClient_PropertyChange(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "IsConnected")
            {
                if (!CiaoClient.IsConnected)
                {
                    this.button1.Text = "Connect";
                    this.groupBox2.Enabled = false;
                    this.textBox7.Enabled = false;
                    PublicKey = null; PrivateKey = null; Key = null;
                }
                else
                {
                    this.button1.Text = "Disconnect";
                    this.groupBox2.Enabled = true;
                    this.textBox7.Enabled = true;
                    if (this.ThisConfiguration.RequestRsaEncryption)
                    {
                        RSACryptoServiceProvider Provider = new RSACryptoServiceProvider(1024);
                        PacketData ToSend = new PacketData();
                        ToSend.DataCategory = "Encryption"; ToSend.DataType = "Request";
                        ToSend.MainData = Provider.ToXmlString(false);
                        Key = Provider.ToXmlString(true);
                        SendEncrypted(ToSend);
                    }
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (!CiaoClient.IsConnected)
            {
                if (IsIP(this.textBox1.Text))
                {
                    CiaoClient.RemoteIP = IPAddress.Parse(this.textBox1.Text);
                }
                else
                {
                    CiaoClient.RemoteIP = Resolve(this.textBox1.Text);
                    if (CiaoClient.RemoteIP == null)
                    {
                        MessageBox.Show(this, "Could not resolve host name", "Error");
                        return;
                    }
                }
                try
                {
                    CiaoClient.Port = Convert.ToInt32(this.textBox2.Text);
                }
                catch
                {
                    MessageBox.Show(this, "Could not parse Port number", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                CiaoClient.Connect();
            }
            else
            {
                CiaoClient.Disconnect();
            }
        }

        public static bool IsIP(String s)
        {
            try
            {
                IPAddress.Parse(s);
            }
            catch
            {
                return false;
            }
            return true;
        }

        public static IPAddress Resolve(String Host)
        {
            IPHostEntry Entry;
            try
            {
                Entry = Dns.GetHostEntry(Host);
            }
            catch (SocketException)
            {
                return null;
            }
            foreach (IPAddress Add in Entry.AddressList)
            {
                if (Add.AddressFamily == AddressFamily.InterNetwork)
                {
                    return Add;
                }
            }
            return null;
        }

        private void menuItem4_Click(object sender, EventArgs e)
        {
            Form AboutForm = new About(this.ProductName, this.ProductVersion);
            AboutForm.ShowDialog(this);
        }

        private void SendEncrypted(Object o)
        {
            Packet p = new Packet(null, o);
            p.Buffer = Encryption.Encrypt(p.Buffer, 1024, PublicKey);
            CiaoClient.Send(p.Buffer);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            PacketData a = new PacketData();
            if (this.button2.Text == "Log in")
            {
                a.DataCategory = "User"; a.DataType = "Request"; a.DataParameters = "LogIn";
                a.MainData = new String[] { this.textBox3.Text, this.textBox4.Text };
            }
            else
            {
                a.DataCategory = "User"; a.DataType = "Request"; a.DataParameters = "LogOut";
            }
            SendEncrypted(a);
        }

        private void label4_Click(object sender, EventArgs e)
        {
            Form form = new SignUpForm();
            form.ShowDialog(this);
        }

        private void menuItem6_Click(object sender, EventArgs e)
        {
            Options OptionsForm = new Options(ThisConfiguration);
            OptionsForm.ShowDialog(this);
            ThisConfiguration = OptionsForm.ThisConfiguration;
            Serializer.SaveObject(ThisConfiguration, "conf.xml", Serializer.FormatterType.SOAP);
        }

        private void MainClientForm_Resize(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized && this.ThisConfiguration.HideWhenMinimized)
            {
                this.Visible = false;
            }
            else
            {
                this.PreviousWindowState = this.WindowState;
            }
        }

        private void TrayIcon_Click(object sender, EventArgs e)
        {
            this.Visible = true;
            this.WindowState = PreviousWindowState;
        }

        private void OnPacketRceived(Packet p)
        {
            PacketData ReceivedData;
            PacketData TempData = new PacketData();

            //Decrypt
            p.Buffer = Encryption.Decrypt(p.Buffer, 1024, PrivateKey);
            try { ReceivedData = (PacketData)p.ObjectBuffer; }
            catch { return; }

            //Proceed
            if (ReceivedData.DataCategory == "Encryption")
            {
                if (ReceivedData.DataType == "Confirmation")
                {
                    PrivateKey = Key;
                }
                else if (ReceivedData.DataType == "Request")
                {
                    if (this.ThisConfiguration.AllowRsaEncryption)
                    {
                        TempData.DataCategory = "Encryption";
                        TempData.DataType = "Confirmation";
                        TempData.MainData = ReceivedData.MainData;
                        SendEncrypted(TempData);
                        PublicKey = (String)ReceivedData.MainData;
                    }
                }
            }
            else if (ReceivedData.DataCategory == "User")
            {
                if (ReceivedData.DataParameters == "LogIn")
                {
                    if (ReceivedData.DataType == "Rejection")
                    {
                        MessageBox.Show(this, "You failed to log in as " + (String)ReceivedData.MainData, "Rejection");
                    }
                    else if (ReceivedData.DataType == "Confirmation")
                    {
                        MessageBox.Show(this, "You are now logged as " + (String)ReceivedData.MainData, "Log in");
                        this.textBox3.Enabled = false;
                        this.textBox4.Enabled = false;
                        this.button2.Text = "Log out";
                    }
                }
                else if (ReceivedData.DataParameters == "LogOut")
                {
                    MessageBox.Show(this, "You have successfully logged out", "Log out");
                    this.textBox3.Enabled = true;
                    this.textBox4.Enabled = true;
                    this.button2.Text = "Log in";
                }
            }
            else if (ReceivedData.DataCategory == "Users")
            {
                if (ReceivedData.DataType == "Notification")
                {
                    PacketData a = new PacketData();
                    a.DataCategory = "Users";
                    a.DataType = "Request";
                    a.DataParameters = "List";
                    SendEncrypted(a);
                }
                else if (ReceivedData.DataType == "Answer")
                {
                    this.listView1.Clear();
                    String[] TempArray = (String[])ReceivedData.MainData;
                    for (int i = 0; i < TempArray.Length; i++)
                    {
                        this.listView1.Items.Add(TempArray[i]);
                    }
                }
            }
            else if (ReceivedData.DataCategory == "Chat")
            {
                if (ReceivedData.DataType == "Notification")
                {
                    this.textBox6.AppendText(((String[])ReceivedData.MainData)[0] + ": " + ((String[])ReceivedData.MainData)[1] + "\r\n");
                }
            }
        }

        private void textBox7_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter && this.textBox7.Text != "")
            {
                PacketData Message = new PacketData();
                Message.DataCategory = "Chat";
                Message.DataType = "Request";
                Message.DataParameters = "Send";
                Message.MainData = textBox7.Text;
                textBox7.Text = "";
                SendEncrypted(Message);
            }
        }

        private void menuItem2_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}