
#ifndef MAILSLOT_H
#define MAILSLOT_H

#include "Package.h"

class MailSlotMessage : public Package
{
public:
    enum MessageType {SYSTEM = 1, TEXT = 2};
public:
    MailSlotMessage();
    MailSlotMessage(const MailSlotMessage & msg);
    MailSlotMessage & operator=(const MailSlotMessage & msg);
    MessageType MsgType();
    void MsgType(MessageType type);
    CString ToUserName();
    void ToUserName(const CString & mailSlotNameStr);
    CString ToComputerName();
    void ToComputerName(const CString & computerNameStr);
    CString FromUserName();
    void FromUserName(const CString & fromStr);
    CString FromComputerName();
    void FromComputerName(const CString & fromStr);
    CString MsgText();
    void MsgText(const CString & msgText);
};

class MailSlot : public CWnd
{
private:
    class MessageQueue 
    {
    public:
        MessageQueue();
        virtual ~MessageQueue();
        void Add(MailSlotMessage & msg);
        bool Get(MailSlotMessage & msg);
        int Count();
        void Reset();
    private:
        CArray<MailSlotMessage, MailSlotMessage> msgAry;
    };   
public:
    MailSlot();
    virtual ~MailSlot();
    BOOL Create(const CString & slotName, CWnd * pParent, UINT notifyID = 0, UINT pollTime = 1000);
    BOOL Read(MailSlotMessage & msg);
    BOOL Send(MailSlotMessage & msg);
   
    CString GetMailSlotPath()  { return slotPath; }
    CString ComputerName()  { return computerName; }
    CString SlotName()      { return slotName; }

protected:
    DECLARE_MESSAGE_MAP()

    BOOL SendToLocal(MailSlotMessage & msg);
    BOOL SendToMailSlot(MailSlotMessage & msg);
    BOOL ReadFromMailSlot();
    void OnTimer(UINT_PTR nIDEvent);
    void NotifyParent();

    // data
    CString slotPath;
    HANDLE mailSlotHandle;
    MessageQueue queue;
    CWnd * pParentWnd;
    UINT_PTR m_nTimer;
    CString computerName;
    CString slotName;
    UINT m_notifyID;
};

#endif
